@@ -160,10 +160,10 @@ public class WebDavWagonTest
     public void testMkdirs()
         throws Exception
     {
-        setupRepositories();
-
         setupWagonTestingFixtures();
 
+        setupRepositories();
+
         WebDavWagon wagon = (WebDavWagon) getWagon();
         wagon.connect( testRepository, getAuthInfo() );
 
@@ -210,10 +210,10 @@ public class WebDavWagonTest
         throws Exception
     {
         // WAGON-244
-        setupRepositories();
-
         setupWagonTestingFixtures();
 
+        setupRepositories();
+
         // reconstruct with no basedir
         testRepository.setUrl(
             testRepository.getProtocol() + "://" + testRepository.getHost() + ":" + testRepository.getPort() );
@@ -255,10 +255,10 @@ public class WebDavWagonTest
     public void testWagonWebDavGetFileList()
         throws Exception
     {
-        setupRepositories();
-
         setupWagonTestingFixtures();
 
+        setupRepositories();
+
         String dirName = "file-list";
 
         String filenames[] =
@@ -325,10 +325,10 @@ public class WebDavWagonTest
     public void testWagonFailsOnPutFailureByDefault()
         throws Exception
     {
-        setupRepositories();
-
         setupWagonTestingFixtures();
 
+        setupRepositories();
+
         File testFile = getTempFile();
 
         System.clearProperty( WebDavWagon.CONTINUE_ON_FAILURE_PROPERTY );
@@ -388,10 +388,10 @@ public class WebDavWagonTest
     public void testWagonContinuesOnPutFailureIfPropertySet()
         throws Exception
     {
-        setupRepositories();
-
         setupWagonTestingFixtures();
 
+        setupRepositories();
+
         File testFile = getTempFile();
 
         String continueOnFailureProperty = WebDavWagon.CONTINUE_ON_FAILURE_PROPERTY;
@@ -523,4 +523,11 @@ public class WebDavWagonTest
 
     }
 
+    @Override
+    protected void verifyWagonExceptionMessage( Exception e, int forStatusCode, String forUrl, String forReasonPhrase )
+    {
+        Repository repo = new Repository( "test-geturl", forUrl );
+        String expectedMessageUrl = ( new WebDavWagon() ).getURL( repo );
+        super.verifyWagonExceptionMessage( e, forStatusCode, expectedMessageUrl, forReasonPhrase );
+    }
 }
